// Example using my interface to stb-truetype, an amazing single file
// truetype font reader, part of the stb library/collection by
// Sean Barrett.

// I am still using the 2013 version since it is small (81k, 60k without comments and examples) and
// it works just fine. I have also tested with the 2021 version.

// By Ingemar Ragnemalm. Experimental version with the revised interface 2023.
// Minimal dependencies! Interfaces to anything!

// Example with the bunny model, to show it working together with other OpenGL code.

#include "MicroGlut.h"
#include "stb_truetype.h"
#include "my-stbtt-lib.h"
// uses framework OpenGL
// uses framework Cocoa
#define MAIN
#include "GL_utilities.h"
#include "LittleOBJLoader.h"
#include "LoadTGA.h"
#include "VectorUtils4.h"
//#include "SimpleGUI.h"

long gWidth=400, gHeight=400;
stbfont *f, *f2, *f3; // The fonts
int count = 0;

GLuint shader;
Model *m;
float v;

void display(void)
{
	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	v += 0.1;
	
	mat4 mtw = T(sin(v), cos(v), 0) *Ry(v);
	uploadMat4ToShader(shader, "modelToWorldMatrix", mtw);
	DrawModel(m, shader, "inPosition", "inNormal", NULL);

	
	my_stbtt_set_color(1, 1, 1);
	my_stbtt_print(40, 100, "my-stbtt works together", f);
	my_stbtt_print(40, 160, "with 3D animations!", f);

	glutSwapBuffers();
}

void reshape(int w, int h)
{
	gWidth = w;
	gHeight = h;
	glViewport(0, 0, w, h);

	printf("reshape to %d %d\n", w, h);
}

void init()
{
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	my_stbtt_init();
	f = my_stbtt_initfont("pdfonts/qumpellkano12.ttf", 64.0);
	f2 = my_stbtt_initfont("pdfonts/linux-libertine.ttf", 32.0);
	f3 = my_stbtt_initfont("pdfonts/schwaben-alt.ttf", 24.0);

	m = LoadModel("bunnyplus2.obj");
	shader = loadShaders("phong.vert", "phong.frag");
	glUseProgram(shader);
	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);

	mat4 projectionMatrix = frustum(-0.5, 0.5, -0.5, 0.5, 0.5, 100);
	uploadMat4ToShader(shader, "projectionMatrix", projectionMatrix);
	mat4 modelToWorldMatrix = IdentityMatrix();
	uploadMat4ToShader(shader, "modelToWorldMatrix", modelToWorldMatrix);
	mat4 modelToWorldMatrix2 = IdentityMatrix();
	uploadMat4ToShader(shader, "modelToWorldMatrix2", modelToWorldMatrix2);
	mat4 worldToViewMatrix = lookAt(3,0,0, 0,0,0, 0,1,0);
	uploadMat4ToShader(shader, "worldToViewMatrix", worldToViewMatrix);
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowSize(840, 600);
	glutInitContextVersion(3, 2);
	glutCreateWindow ("my-stbtt example with the bunny");
	glutDisplayFunc(display); 
	glutReshapeFunc(reshape);
	glutRepeatingTimer(20);
	init ();
	reshape(400, 400);
	glutMainLoop();
}
